/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap;

import static org.ow2.petals.binding.soap.Constants.Axis2.AXIS2_XML;
import static org.ow2.petals.binding.soap.Constants.Axis2.MODULES_PATH;
import static org.ow2.petals.binding.soap.Constants.Axis2.SERVICES_PATH;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;

import org.apache.commons.io.FileUtils;

import org.ow2.petals.component.framework.DefaultBootstrap;

/**
 * The bootstrap class. Copy all the configuration files in the good
 * directories.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class SoapBootstrap extends DefaultBootstrap {

    protected SoapBootstrapOperations bootstrapOperations;

    // A logger must be static
    // (otherwise the garbage collector remove it from the memory)
    public static final Logger logger = Logger.getLogger("org.apache.axis2");

    /**
     * Add a module to the Axis2 engine. This method is used by the
     * configuration MBean.
     * 
     * @param url
     * @return
     * @throws JBIException
     */
    public String addModule(final String url) throws Exception {
        return bootstrapOperations.addModule(url);
    }

    /**
     * Copy the needed configuration files to the Axis2 directories. Do not copy
     * them if the files are already present (the install phase has already been
     * done and we are now in the uninstall phase).
     * 
     */
    protected void copyConfigurationFiles() throws JBIException {

        final File installRootFile = new File(getInstallContext().getInstallRoot());

        /* copy required files from META-INF directory */
        final File axis2File = new File(installRootFile, "META-INF" + File.separator + AXIS2_XML);

        // copy axis2.xml file
        if (axis2File.exists()) {
            final String workspaceRootDir = installContext.getContext().getWorkspaceRoot();
            final File destFile = new File(workspaceRootDir, axis2File.getName());
            if (!destFile.exists()) {
                try {
                    FileUtils.copyFile(axis2File, destFile);
                } catch (final IOException e) {
                    throw new JBIException(
                            "Can not copy the axis2 configuration file, axis2 can not be used", e);
                }
            }
        } else {
            throw new JBIException("the axis2 configuration file do not exist at '+ "
                    + axis2File.getAbsolutePath() + "', axis2 can not be used");
        }

        // copy modules from component root path
        final String workspaceRootDir = getInstallContext().getContext().getWorkspaceRoot();
        final File modules = new File(workspaceRootDir, MODULES_PATH);
        final File[] moduleFiles = installRootFile.listFiles(new BootStrapFileFilter());
        for (final File file : moduleFiles) {
            final File destFile = new File(modules, file.getName());
            if (!destFile.exists()) {
                try {
                    FileUtils.copyFile(file, destFile);
                } catch (final IOException e) {
                    getLogger().warning(
                            "The module '" + file.getName()
                                    + "' can not been copied to modules directory");
                }
            }
        }
    }

    /**
     * Create the directories that will be used by axis
     * 
     * @param installPath
     */
    protected void createWorkDirectories() {
        final String workspaceRootDir = installContext.getContext().getWorkspaceRoot();
        final File modules = new File(workspaceRootDir, MODULES_PATH);
        final File services = new File(workspaceRootDir, SERVICES_PATH);
        if (!modules.exists()) {
            if (!modules.mkdirs()) {
                getLogger().warning(
                        "Cannot create modules directory at '" + modules.getAbsolutePath() + "'");
            }
        }
        if (!services.exists()) {
            if (!services.mkdirs()) {
                getLogger().warning(
                        "Cannot create services directory at '" + services.getAbsolutePath() + "'");
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.DefaultBootstrap#doInit()
     */
    @Override
    protected void doInit() throws JBIException {
        createWorkDirectories();
        copyConfigurationFiles();

        // Disable Axis 2 logging
        logger.setLevel(Level.OFF);

        if (bootstrapOperations == null) {
            bootstrapOperations = new SoapBootstrapOperations(installContext);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.DefaultBootstrap#getAttributeList()
     */
    @Override
    protected List<String> getAttributeList() {
        final List<String> attributes = new ArrayList<String>();
        attributes.add("httpPort");
        attributes.add("httpHostName");
        attributes.add("httpServicesList");
        attributes.add("httpServicesContext");
        attributes.add("httpServicesMapping");
        attributes.add("httpThreadPoolSizeMin");
        attributes.add("httpThreadPoolSizeMax");
        attributes.add("httpAcceptors");
        attributes.add("javaNamingFactoryInitial");
        attributes.add("javaNamingProviderURL");
        attributes.add("jmsConnectionFactoryJNDIName");

        return attributes;
    }

    /**
     * Get HttpAcceptors.
     * 
     * @return
     */
    public int getHttpAcceptors() {
        int httpAcceptors = 0;

        final String httpAcceptorsString = bootstrapOperations.getParam(
                Constants.HttpServer.HTTP_ACCEPTORS, getJbiComponentConfiguration());
        if (httpAcceptorsString != null) {
            httpAcceptors = Integer.parseInt(httpAcceptorsString);
        }

        return httpAcceptors;
    }

    /**
     * Get the HTTP host name
     * 
     * @return
     */
    public String getHttpHostName() {
        return bootstrapOperations.getParam(Constants.HttpServer.HTTP_HOSTNAME,
                getJbiComponentConfiguration());
    }

    /**
     * Get Http port.
     * 
     * @return
     */
    public int getHttpPort() {
        int httpPort = 0;
        final String httpPortString = bootstrapOperations.getParam(Constants.HttpServer.HTTP_PORT,
                getJbiComponentConfiguration());
        if (httpPortString != null) {
            httpPort = Integer.parseInt(httpPortString);
        }

        return httpPort;
    }

    /**
     * Get HTTP services context.
     * 
     * @return
     */
    public String getHttpServicesContext() {
        return bootstrapOperations.getParam(Constants.HttpServer.HTTP_SERVICES_CONTEXT,
                getJbiComponentConfiguration());
    }

    /**
     * Get HTTP services list.
     * 
     * @return
     */
    public boolean getHttpServicesList() {
        boolean httpServicesList = true;

        final String httpServicesListString = bootstrapOperations.getParam(
                Constants.HttpServer.HTTP_SERVICES_LIST, getJbiComponentConfiguration());
        if (httpServicesListString != null) {
            httpServicesList = Boolean.parseBoolean(httpServicesListString);
        }

        return httpServicesList;
    }

    /**
     * Get HTTP services Mapping.
     * 
     * @return
     */
    public String getHttpServicesMapping() {
        return bootstrapOperations.getParam(Constants.HttpServer.HTTP_SERVICES_MAPPING,
                getJbiComponentConfiguration());
    }

    /**
     * Get HTTP thread pool maximum size
     * 
     * @return
     */
    public int getHttpThreadPoolSizeMax() {
        int httpThreadPoolSizeMax = 0;

        final String httpThreadPoolSizeMaxString = bootstrapOperations.getParam(
                Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MAX, getJbiComponentConfiguration());
        if (httpThreadPoolSizeMaxString != null) {
            httpThreadPoolSizeMax = Integer.parseInt(httpThreadPoolSizeMaxString);
        }

        return httpThreadPoolSizeMax;
    }

    /**
     * Get HTTP thread pool minimum size
     * 
     * @return
     */
    public int getHttpThreadPoolSizeMin() {
        int httpThreadPoolSizeMin = 0;

        final String httpThreadPoolSizeMinString = bootstrapOperations.getParam(
                Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MIN, getJbiComponentConfiguration());
        if (httpThreadPoolSizeMinString != null) {
            httpThreadPoolSizeMin = Integer.parseInt(httpThreadPoolSizeMinString);
        }

        return httpThreadPoolSizeMin;
    }

    /**
     * Get Java naming factory initial
     * 
     * @return
     */
    public String getJavaNamingFactoryInitial() {
        return bootstrapOperations.getParam(Constants.JmsTransportLayer.JNDI_INITIAL_FACTORY,
                getJbiComponentConfiguration());
    }

    /**
     * Get Java naming provider URL
     * 
     * @return
     */
    public String getJavaNamingProviderURL() {
        return bootstrapOperations.getParam(Constants.JmsTransportLayer.JNDI_PROVIDER_URL,
                getJbiComponentConfiguration());
    }

    /**
     * Get JMS connection factory JNDI name
     * 
     * @return
     */
    public String getJMSConnectionFactoryJNDIName() {
        return bootstrapOperations.getParam(Constants.JmsTransportLayer.CONFAC_JNDINAME,
                getJbiComponentConfiguration());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.DefaultBootstrap#getMethodList()
     */
    @Override
    protected List<String> getMethodList() {
        final List<String> methods = new ArrayList<String>();
        // expose addModule for management
        methods.add("addModule");

        return methods;
    }

    /**
     * Set HTTP acceptors.
     * 
     * @param httpAcceptors
     */
    public void setHttpAcceptors(final int httpAcceptors) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_ACCEPTORS, Integer
                .toString(httpAcceptors), getJbiComponentConfiguration());
    }

    /**
     * Set the HTTP host name.
     * 
     * @param httpHostName
     */
    public void setHttpHostName(final String httpHostName) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_HOSTNAME, httpHostName,
                getJbiComponentConfiguration());
    }

    /**
     * Set the HTTP port.
     * 
     * @param httpPort
     */
    public void setHttpPort(final int httpPort) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_PORT, Integer.toString(httpPort),
                getJbiComponentConfiguration());
    }

    /**
     * Set HTTP services context.
     * 
     * @param httpServicesContext
     */
    public void setHttpServicesContext(final String httpServicesContext) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_SERVICES_CONTEXT,
                httpServicesContext, getJbiComponentConfiguration());
    }

    /**
     * Set HTTP services list.
     * 
     * @param httpServicesList
     */
    public void setHttpServicesList(final boolean httpServicesList) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_SERVICES_LIST, Boolean
                .toString(httpServicesList), getJbiComponentConfiguration());
    }

    /**
     * Set HTTP services mapping.
     * 
     * @param httpServicesMapping
     */
    public void setHttpServicesMapping(final String httpServicesMapping) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_SERVICES_MAPPING,
                httpServicesMapping, getJbiComponentConfiguration());
    }

    /**
     * Set HTTP thread pool maximum size.
     * 
     * @param httpThreadPoolSizeMax
     */
    public void setHttpThreadPoolSizeMax(final int httpThreadPoolSizeMax) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MAX, Integer
                .toString(httpThreadPoolSizeMax), getJbiComponentConfiguration());
    }

    /**
     * Set HTTP thread pool minimum size.
     * 
     * @param httpThreadPoolSizeMin
     */
    public void setHttpThreadPoolSizeMin(final int httpThreadPoolSizeMin) {
        bootstrapOperations.setParam(Constants.HttpServer.HTTP_THREAD_POOL_SIZE_MIN, Integer
                .toString(httpThreadPoolSizeMin), getJbiComponentConfiguration());
    }

    /**
     * Set Java naming factory initial
     * 
     * @param javaNamingFactoryInitial
     */
    public void setJavaNamingFactoryInitial(final String javaNamingFactoryInitial) {
        bootstrapOperations.setParam(Constants.JmsTransportLayer.JNDI_INITIAL_FACTORY,
                javaNamingFactoryInitial, getJbiComponentConfiguration());
    }

    /**
     * Set Java naming provider URL
     * 
     * @param javaNamingProviderURL
     */
    public void setJavaNamingProviderURL(final String javaNamingProviderURL) {
        bootstrapOperations.setParam(Constants.JmsTransportLayer.JNDI_PROVIDER_URL,
                javaNamingProviderURL, getJbiComponentConfiguration());
    }

    /**
     * Set JMS connection factory JNDI name
     * 
     * @param jmsConnectionFactoryJNDIName
     */
    public void setJMSConnectionFactoryJNDIName(final String jmsConnectionFactoryJNDIName) {
        bootstrapOperations.setParam(Constants.JmsTransportLayer.CONFAC_JNDINAME,
                jmsConnectionFactoryJNDIName, getJbiComponentConfiguration());
    }
}
