/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener.incoming;

import static org.ow2.petals.binding.soap.Constants.SOAP.FAULT_CLIENT;
import static org.ow2.petals.binding.soap.Constants.SOAP.FAULT_SERVER;

import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.util.XMLUtils;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.BindingOperation;
import org.ow2.easywsdl.wsdl.api.binding.BindingProtocol.SOAPMEPConstants;
import org.w3c.dom.Element;

import org.ow2.petals.binding.soap.util.SUPropertiesHelper;
import org.ow2.petals.binding.soap.util.WsdlHelper;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Processor to handle incoming SOAP messages.
 * 
 * Created on 14 févr. 08
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since
 * 
 */
public class SOAPProcessor extends RequestProcessor {

    /**
     * 
     * @param component
     * @param channel
     * @param log
     */
    public SOAPProcessor(final AbstractComponent component, final DeliveryChannel channel,
            final Logger log) {
        super(component, channel, log);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.binding.soap.listener.incoming.RequestProcessor#
     * getProcessorName()
     */
    @Override
    protected String getProcessorName() {
        return "SOAP";
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.binding.soap.listener.incoming.RequestProcessor#process
     * (org.apache.axis2.context.MessageContext,
     * org.apache.axiom.soap.SOAPFactory)
     */
    @Override
    protected MessageContext process(final MessageContext inContext, final SOAPFactory factory)
            throws AxisFault, UnsupportedEncodingException {
        getLogger().log(Level.FINE, "Processing the incoming SOAP message");

        // Type of service has been checked before
        final PetalsAxisService axisService = (PetalsAxisService) inContext.getAxisService();

        // Retrieve message operation and service
        final String soapAction = inContext.getWSAAction();

        getLogger().log(Level.FINEST, "soapAction found in the incoming message : " + soapAction);

        QName jbiOperation = null;
        MEPType mep = null;
        BindingOperation bindingOperation = retrieveSOAPBindingOperation(inContext,
				soapAction, axisService);
        if(bindingOperation != null)
        {
        	// get the operation name
        	jbiOperation = bindingOperation.getOperation().getQName();
	        if (getLogger().isLoggable(Level.FINEST)) {
	            getLogger().log(
	                    Level.FINEST,
	                    "jbiOperation retrieved : {" + jbiOperation.getNamespaceURI() + "}"
	                            + jbiOperation.getLocalPart());
	        }
	        
	        // get the MEP
	        SOAPMEPConstants mepFromBindingOperation = bindingOperation.getMEP();
	        if(mepFromBindingOperation == SOAPMEPConstants.ONE_WAY) {
	        	mep = MEPType.IN_ONLY;
	        }
	        else if(mepFromBindingOperation == SOAPMEPConstants.REQUEST_RESPONSE) {
	        	mep = MEPType.IN_OUT;
	        }
        }
        else
        {
            // To be removed when the new mechanism (findOperation() upstairs)
            // has been improved!!!:
            jbiOperation = retrieveJBIOperationUsingOldMechanism(inContext,
					soapAction);
        }
        
        if (jbiOperation == null) {
            throw new AxisFault("Operation not found in message context", FAULT_CLIENT);
        }
        
    	// if MEP was not found in the WSDL, get mep from the extensions
        if(mep == null)
        {
            mep = axisService.getConsumes().getMep();
        }

        // create the message exchange
        MessageExchange exchange = null;
        try {
            exchange = createMessageExchange(inContext, jbiOperation, mep, axisService);
        } catch (final AxisFault e) {
            final String errorMsg = "Error while transforming SOAP request to JBI MessageExchange";
            getLogger().log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, FAULT_SERVER, e);
        } catch (final MessagingException e) {
            final String errorMsg = "Error while transforming SOAP request to JBI MessageExchange";
            getLogger().log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, FAULT_SERVER, e);
        }

        // Set the soap action jbi property.
        exchange.setProperty("soap-action", soapAction);

        // get timeout
        final long timeout = SUPropertiesHelper.retrieveTimeout(axisService
                .getConsumesCDKExtensions());
        
        // send the message exchange through JBI
        // As the SOAP/HTTP request may wait for a response,
        // use the sendSynchrone MEP
        boolean sent = false;
        try {

            if (timeout == -1L) {
                sent = getChannel().sendSync(exchange);
            } else {
                sent = getChannel().sendSync(exchange, timeout);
            }

        } catch (final MessagingException e) {
            final String errorMsg = "Error while sending JBI exchange with id '"
                    + exchange.getExchangeId() + "'";
            getLogger().log(Level.WARNING, errorMsg, e);
            throw new AxisFault(errorMsg, FAULT_SERVER, e);
        }

        if (!sent) {
            final String errorMsg = "A timeout occurs on JBI exchange with id '"
                    + exchange.getExchangeId() + "'";
            getLogger().log(Level.WARNING, errorMsg);
            throw new AxisFault(errorMsg, FAULT_SERVER);
        }
        
        // process the response received from JBI NMR. The out context is
        // returned
        return processJBIResponse(exchange, inContext, factory);
    }


	/**
	 * Retrieve the SOAP binding operation from WSA Addressing, 
	 * 		then if not found, from SOAP body
	 * 
	 * @param inContext	SOAP request context       
	 * @param soapAction SOAP action	
	 * @param axisService AXIS service which manages the request		   
	 * 
	 * @return the bing opeation if found, otherwise null
	 * @throws AxisFault
	 */
	private BindingOperation retrieveSOAPBindingOperation(
			final MessageContext inContext, final String soapAction,
			final PetalsAxisService axisService) throws AxisFault {
		final Description desc = axisService.getDescription();

        BindingOperation bindingOperation = null;
		if(desc != null)
		{
    		if (soapAction != null) {
    		    bindingOperation = WsdlHelper.findOperationUsingSoapAction(soapAction, desc);
    		}
    
    		if (bindingOperation == null) {
    			try
    			{
    			    final Element firstElement = XMLUtils.toDOM(inContext.getEnvelope().getBody()
    			            .getFirstElement());
    			    final String endpointName = axisService.getConsumes().getEndpointName();
    			    final QName serviceName = axisService.getConsumes().getServiceName();
    			    bindingOperation = WsdlHelper.findOperationUsingElement(firstElement, desc, endpointName,
    			            serviceName);
    			}
    			catch(Exception e) {}
    		}
		}
		
		return bindingOperation;
	}
	
    /**
     * Retrieve operation from the old mechanism
     * 
     * @param inContext SOAP request context 
     * @param soapAction SOAP action	
     * 			  
     * @return operation as QName
     */
	private QName retrieveJBIOperationUsingOldMechanism(
			final MessageContext inContext, final String soapAction) {
		QName jbiOperation;
		if (!StringHelper.isNullOrEmpty(soapAction)) {
		    final String namespace = soapAction.substring(0, inContext.getWSAAction()
		            .lastIndexOf('/') + 1);
		    final String opname = inContext.getWSAAction().substring(
		            inContext.getWSAAction().lastIndexOf('/') + 1);
		    final QName operation = new QName(namespace, opname);
		    jbiOperation = operation;
		} else {
			jbiOperation = retrieveOperationFromSOAPBody(inContext);
		}

		if (getLogger().isLoggable(Level.FINEST)) {
		    getLogger().log(
		            Level.FINEST,
		            "jbiOperation retrieved using old mechanism : {"
		                    + jbiOperation.getNamespaceURI() + "}"
		                    + jbiOperation.getLocalPart());
		}
		return jbiOperation;
	}
}
